

###########this part of the code simulates exact spreading model over the trmporal network###################



# here we load the functions required for the simulation

source("NeighborhoodDataWD.R");
source("Net_Import.R");
source("GEMF_SIM_several_switching_exact_rev.R");
source("Para_switching_exact.R");



# here we load two layer networks in a format that the main simulation function uses

N=500;
Net1=Net_Import("L1er.txt",N);
Net2=Net_Import("L2er.txt",N);
v1=Net1[[2]]; v2=Net1[[3]];
IN1=rbind(v1,v2); Nei1=Net1[[1]][[1]];

v1=Net2[[2]]; v2=Net2[[3]];
IN2=rbind(v1,v2); Nei2a=Net2[[1]][[1]];

Nei2b=Nei2a*0;
for (n in 1:N){
  if(IN2[1,n]!=0){
    for(ind in IN2[1,n]:IN2[2,n]){
      neigbor<-Nei2a[1,ind];
      
      neisofneighbor<-Nei2a[1,IN2[1,neigbor]:IN2[2,neigbor]];
      corind=(which(neisofneighbor == n)[[1]])-1+IN2[1,neigbor];
      Nei2b[1,ind]<-corind;
    }
    
  }
  
}
Nei2=rbind(Nei2a,Nei2b)
rm(v1,v2,Net1,Net2,Nei2a,Nei2b)



#we set initial sates of the nodes
M=4;
x0=matrix(as.integer(4),1,N);
 


#simulation terminator 
maxNumevent=1000000;Runtime=150;


#here we set the model parameters(rates) and arranged them in a format that is used by the main simulation
p2=0.5;
p0=0.5;
bet=0.24;
gam1=5;
gam2=gam1*((1/p2)-1);
Para=Para_switching_exact(delta=1,gamma_1=gam1,gamma_2=gam2,beta1=bet,beta2=bet,p0);



# here we run the simulation 200 times and save the result as an R file
lst<- GEMF_SIM_several_switching_exact_rev(Para,IN1,IN2,Nei1,Nei2,x0,maxNumevent,Runtime,N,numrun=200,res=1);  
filename=paste("beta",toString(bet),"gamma1",toString(gam1),"p2",toString(p2),"run_",toString(200),".RData",sep="");
save(lst,file=filename)



#load files and save it as text file

run=200;

  filename=paste("beta",toString(beta),"gamma1",toString(gamma1),"p2",toString(p2),"run_",toString(run),".RData",sep="");
  load(filename);
 
  T=lst[[1]]; 
  data=lst[[5]]+lst[[4]];
  EI=cbind(T,data);
  filename=paste("beta",toString(beta),"gamma1",toString(gamma1),"p2",toString(p2),
                 "EI",".txt",sep="");
  write(EI, file = filename,ncolumns=dim(EI)[1],sep = "\t ")
  
#--in the text file first row is time and each subsequent row shows total 
#infected individuals in one run of the simulation at different time instants.









###########this part of the code simulates Markov model ###################



# here we load the functions required for the simulation

#( comprehensive explaniation of these functions is available at
#https://www.ece.k-state.edu/netse/files/Rtutorial.pdf)

source("NeighborhoodDataWD.R");
source("Net_Import.R");
source("NetCmbn.R");
source("Post_Population.R");
source("Post_NodesState.R");
source("GEMF_SIM.R");
source("GEMF_SIM_Prob.R");
source("GEMF_SIM_several.R");
source("Para_switching.R");


# here we load two layer networks in a format that the main simulation function uses
N=500;
Net1=Net_Import("L1er.txt",N);
Net2=Net1;
Net3=Net_Import("L2er.txt",N);
NetSet=list(Net1,Net2,Net3);
Net=NetCmbn(NetSet,N);
rm(Net1,Net2,Net3,NetSet);


#here we set the model parameters(rates) and arranged them in a format that is used by the main simulation
p2=0.5;
p0=0.5;
bet=0.24;
gam1=5;
gam2=gam1*((1/p2)-1);
Para=Para_switching(delta=1,gamma_1=gam1,gamma_2=gam2,bet,betprim=bet*p0)
M<-Para[[1]];


#we set initial sates of the nodes
x0=matrix(as.integer(4),1,N);


#simulation terminator 
maxNumevent=1000000;Runtime=150;


# here we run the simulation 200 times and save the result as an R file
dd={};
for (i in 1:200){
  lst<-GEMF_SIM(Para,Net,x0,maxNumevent,Runtime,N);
  dd[[i]]=lst;
  print(i);
}


filename=paste("beta",toString(bet),"gamma1",toString(gam1),"p2",toString(p2),"run_",toString(200),".RData",sep="");
save(dd,file=filename)


#load file and save the result as a text file

source("Post_NodesState.R");

Tr<-seq(0,150,0.5);
EI=Tr;

for(run in 1:1){
  filename=paste("beta",toString(bet),"gamma1",toString(gam1),"p2",toString(p2),"run_",toString(200),".RData",sep="");
  load(filename);
  numrun=length(dd);
  
  for(runn in 1:numrun){
    
    ts<-dd[[runn]][[1]];
    n_index<-dd[[runn]][[2]];
    i_index<-dd[[runn]][[3]];
    j_index<-dd[[runn]][[4]];
    Tf<-dd[[runn]][[5]];
    lasteventnumber<-dd[[runn]][[6]]; 
    lst3<-Post_NodesState(x0,M,N,ts,n_index,j_index,lasteventnumber,timstp=0.5,Runtime=150)
    nodstt=lst3[[2]];
    regI=rowSums(nodstt>=3);
    EI=rbind(EI,regI);
    
  };  
};


filename=paste("beta",toString(beta),"gamma1",toString(gamma1),"p2",toString(p2),
               "EI",".txt",sep="");
write(EI, file = filename,ncolumns=dim(EI)[1],sep = "\t ")

#--in the text file first column is time points and each subsequent column shows total 
#infected individuals in one run of the simulation at different time instants.







########this part of the code solve the N-intertwined differential equations of the spreading model#######

# here we load the functions required for the simulation
source("NeighborhoodDataWD.R");
source("Net_Import.R");
source("NetCmbn.R");
source("Para_switching.R");
source("GEMF_ODE.R");


# here we load two layer networks in a format that the main solver function uses
N=500;
Net1=Net_Import("L1er.txt",N);
Net2=Net1;
Net3=Net_Import("L2er.txt",N);
NetSet=list(Net1,Net2,Net3);
Net=NetCmbn(NetSet,N);
rm(Net1,Net2,Net3,NetSet);

#here we set the model parameters(rates) and arranged them in a format that is used by the main solver
p2=0.5;
p0=0.5;
bet=0.24;
gam1=5;
gam2=gam1*((1/p2)-1);
Para=Para_switching(delta=1,gamma_1=gam1,gamma_2=gam2,bet,betprim=bet*p0)
M<-Para[[1]];


#we set initial sates of the nodes
x0=matrix(as.integer(4),1,N); 

P0=matrix(0,M,N);
for(i in 1:N ){
  P0[x0[i],i]=1;
}

#here we solve the N-intertwined equation
TT=150;# final time
sol=GEMF_ODE(Para,Net,P0,TT); # solving ODE


#here we plot the results
t=sol[[1]];  # time points that we have mean field solution for them.
P_t=sol[[2]]; #P_t is a list of M (number of state) elements. each element is a 
# n_t*N matrix (number of time points* number of nodes). for example P_t[[m]] 
#is a matrix that stores probablity of finding nodes in state m at 
#different time points.  Probablity of finding node n in state m at 
#time point j, is  p_t[[m]][j,n]

#ploting population of each comportmant through time 
Comp_P_t=matrix(0,size(t)[2],M)
leg=character(M)

for (i in 1:M){
  Comp_P_t[,i]=rowSums(P_t[[i]]);
  
  leg[i]= paste("s",toString(i));
}


matplot(t, Comp_P_t, type = "l", lty = 1,lwd=2,  xlab="time",ylab="states' population",col= rainbow(M))
legend("topright",legend=leg,lty = 1,lwd=2,col= rainbow(M))  







